@@ -5,14 +5,8 @@
 
 """Parse SQL statements."""
 
-__version__ = '0.1.0'
 
-import logging
-import os
-
-
-if 'SQLPARSE_DEBUG' in os.environ:
-    logging.basicConfig(level=logging.DEBUG)
+__version__ = '0.1.2'
 
 
 class SQLParseError(Exception):
@@ -59,4 +53,3 @@ def split(sql):
     stack = engine.FilterStack()
     stack.split_statements = True
     return [unicode(stmt) for stmt in stack.run(sql)]
-
