@@ -30,7 +30,7 @@ class TestFormat(TestCaseBase):
                           identifier_case='foo')
         sql = 'select * from "foo"."bar"'
         res = sqlparse.format(sql, identifier_case="upper")
-        self.ndiffAssertEqual(res, 'select * from "FOO"."BAR"')
+        self.ndiffAssertEqual(res, 'select * from "foo"."bar"')
 
     def test_strip_comments_single(self):
         sql = 'select *-- statement starts here\nfrom foo'
@@ -104,6 +104,14 @@ class TestFormatReindent(TestCaseBase):
                                                'select *',
                                                'from bar;']))
 
+    def test_keywords_between(self):  # issue 14
+        # don't break AND after BETWEEN
+        f = lambda sql: sqlparse.format(sql, reindent=True)
+        s = 'and foo between 1 and 2 and bar = 3'
+        self.ndiffAssertEqual(f(s), '\n'.join(['',
+                                               'and foo between 1 and 2',
+                                               'and bar = 3']))
+
     def test_parenthesis(self):
         f = lambda sql: sqlparse.format(sql, reindent=True)
         s = 'select count(*) from (select * from foo);'
@@ -159,12 +167,23 @@ class TestFormatReindent(TestCaseBase):
                                                'from a,',
                                                '     b']))
 
+    def test_identifier_list_with_functions(self):
+        f = lambda sql: sqlparse.format(sql, reindent=True)
+        s = ("select 'abc' as foo, coalesce(col1, col2)||col3 as bar,"
+             "col3 from my_table")
+        self.ndiffAssertEqual(f(s), '\n'.join(
+            ["select 'abc' as foo,",
+             "       coalesce(col1, col2)||col3 as bar,",
+             "       col3",
+             "from my_table"]))
+
     def test_case(self):
         f = lambda sql: sqlparse.format(sql, reindent=True)
         s = 'case when foo = 1 then 2 when foo = 3 then 4 else 5 end'
-        self.ndiffAssertEqual(f(s), '\n'.join(['case when foo = 1 then 2',
-                                               '     when foo = 3 then 4',
-                                               '     else 5',
+        self.ndiffAssertEqual(f(s), '\n'.join(['case',
+                                               '    when foo = 1 then 2',
+                                               '    when foo = 3 then 4',
+                                               '    else 5',
                                                'end']))
 
     def test_nested_identifier_list(self):  # issue4
