@@ -1,3 +1,4 @@
+
 # -*- coding: utf-8 -*-
 
 # Tests splitting functions.
@@ -52,6 +53,16 @@ class SQLSplitTest(TestCaseBase):
         self.assertEqual(len(stmts), 3)
         self.ndiffAssertEqual(''.join(unicode(q) for q in stmts), sql)
 
+    def test_dashcomments_eol(self):
+        stmts = sqlparse.parse('select foo; -- comment\n')
+        self.assertEqual(len(stmts), 1)
+        stmts = sqlparse.parse('select foo; -- comment\r')
+        self.assertEqual(len(stmts), 1)
+        stmts = sqlparse.parse('select foo; -- comment\r\n')
+        self.assertEqual(len(stmts), 1)
+        stmts = sqlparse.parse('select foo; -- comment')
+        self.assertEqual(len(stmts), 1)
+
     def test_begintag(self):
         sql = load_file('begintag.sql')
         stmts = sqlparse.parse(sql)
@@ -86,3 +97,9 @@ class SQLSplitTest(TestCaseBase):
                'comment on table actor is \'The actor table.\';')
         stmts = sqlparse.split(sql)
         self.assertEqual(len(stmts), 2)
+
+    def test_cursor_declare(self):
+        sql = ('DECLARE CURSOR "foo" AS SELECT 1;\n'
+               'SELECT 2;')
+        stmts = sqlparse.split(sql)
+        self.assertEqual(len(stmts), 2)
